
import { createRoot } from 'react-dom/client'
import { HelmetProvider } from 'react-helmet-async'
import { lazy, Suspense, useState, useEffect } from 'react'

// Priorizar o componente App para carregamento inicial
import App from './App.tsx'

// Componente de contexto de autenticação carregado sob demanda
const AuthProvider = lazy(() => 
  import(/* webpackChunkName: "auth-provider" */ './hooks/use-auth.tsx')
  .then(module => ({ default: module.AuthProvider }))
)

// Melhor componente de carregamento com footprint mínimo
const AppLoader = () => (
  <div className="loading-pulse" aria-label="Loading">
    <div></div>
    <div></div>
    <div></div>
  </div>
)

// Declare type for NetworkInformation interface
interface NetworkInformation {
  saveData: boolean;
  effectiveType: string;
}

// Declare type for Navigator with connection property
interface NavigatorWithConnection extends Navigator {
  connection?: NetworkInformation;
}

// Componente Root otimizado com hidratação progressiva
const RootComponent = () => {
  const [isHydrated, setIsHydrated] = useState(false);
  const [isStylesLoaded, setIsStylesLoaded] = useState(false);

  useEffect(() => {
    // Check if styles are loaded
    const styleSheets = document.querySelectorAll('link[rel="stylesheet"]');
    let loadedCount = 0;
    
    const checkStylesLoaded = () => {
      loadedCount++;
      if (loadedCount >= styleSheets.length) {
        setIsStylesLoaded(true);
      }
    };

    styleSheets.forEach(sheet => {
      if ((sheet as HTMLLinkElement).sheet) {
        checkStylesLoaded();
      } else {
        sheet.addEventListener('load', checkStylesLoaded, { once: true });
      }
    });

    // Mark hydration as complete using requestIdleCallback
    if ('requestIdleCallback' in window) {
      window.requestIdleCallback(() => {
        setIsHydrated(true);
      }, { timeout: 500 }); // Shorter timeout for faster rendering
    } else {
      // Fallback for browsers that don't support requestIdleCallback
      setTimeout(() => {
        setIsHydrated(true);
      }, 10); // Small timeout to yield to the main thread
    }

    return () => {
      styleSheets.forEach(sheet => {
        sheet.removeEventListener('load', checkStylesLoaded);
      });
    };
  }, []);

  // Add class to document when app is fully hydrated
  useEffect(() => {
    if (isHydrated && isStylesLoaded) {
      document.documentElement.classList.add('app-ready');
    }
  }, [isHydrated, isStylesLoaded]);

  return (
    <HelmetProvider>
      <Suspense fallback={<AppLoader />}>
        <AuthProvider>
          <App />
        </AuthProvider>
      </Suspense>
    </HelmetProvider>
  );
};

// Criar raiz e renderizar com priorização de recursos críticos
const rootElement = document.getElementById("root");
if (!rootElement) throw new Error('Failed to find the root element');

// Measure initial load performance
if (process.env.NODE_ENV === 'production') {
  // Reportar métricas de desempenho
  setTimeout(() => {
    if ('performance' in window && 'getEntriesByType' in performance) {
      const perfEntries = performance.getEntriesByType('navigation');
      if (perfEntries.length > 0) {
        const timing = perfEntries[0] as PerformanceNavigationTiming;
        console.log('Time to first byte:', timing.responseStart);
        console.log('DOM Content Loaded:', timing.domContentLoadedEventEnd);
        console.log('Load event:', timing.loadEventEnd);
      }
    }
  }, 0);
}

// Renderização imediata para conteúdo crítico
createRoot(rootElement).render(<RootComponent />);

// Prefetch low-priority resources only when browser is idle
if ('requestIdleCallback' in window) {
  window.requestIdleCallback(() => {
    // Only prefetch components on fast connections
    const navigatorWithConnection = navigator as NavigatorWithConnection;
    const hasConnection = navigatorWithConnection.connection !== undefined;
    
    // Default to false when on slow connections or saveData is true
    const isFastConnection = hasConnection ? 
      !navigatorWithConnection.connection?.saveData && 
      (navigatorWithConnection.connection?.effectiveType ? 
        ['4g', '5g'].includes(navigatorWithConnection.connection.effectiveType) : 
        true) :
      true;
    
    if (isFastConnection) {
      // Prefetch components that will likely be needed soon but aren't critical
      const prefetchComponents = [
        import(/* webpackChunkName: "footer" */ './components/Footer.tsx'),
        import(/* webpackChunkName: "feature-section" */ './components/FeatureSection.tsx')
      ];
      
      // Execute prefetches
      Promise.all(prefetchComponents).catch(() => {
        // Silently handle errors in prefetching - it's just an optimization
      });
    }
  }, { timeout: 2000 });
}
